#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using System;

namespace HeuristicLab.Problems.DataAnalysis {
  /// <summary>
  /// Abstract base class for classification data analysis solutions
  /// </summary>
  [StorableClass]
  public abstract class ClassificationSolution : DataAnalysisSolution, IClassificationSolution {
    private const string ThresholdsResultsName = "Thresholds";
    [StorableConstructor]
    protected ClassificationSolution(bool deserializing) : base(deserializing) { }
    protected ClassificationSolution(ClassificationSolution original, Cloner cloner)
      : base(original, cloner) {
    }
    public ClassificationSolution()
      : base() {
      DoubleArray thresholds = new DoubleArray();
      Add(new Result(ThresholdsResultsName, "The threshold values for class boundaries.", thresholds));
      thresholds.Reset += new EventHandler(thresholds_Reset);
      thresholds.ItemChanged += new EventHandler<EventArgs<int>>(thresholds_ItemChanged);
    }

    #region IClassificationSolution Members

    public new IClassificationModel Model {
      get { return (IClassificationModel)base.Model; }
    }

    public new IClassificationProblemData ProblemData {
      get { return (IClassificationProblemData)base.ProblemData; }
    }

    public virtual IEnumerable<double> EstimatedValues {
      get {
        return GetEstimatedValues(Enumerable.Range(0, ProblemData.Dataset.Rows));
      }
    }

    public virtual IEnumerable<double> EstimatedTrainingValues {
      get {
        return GetEstimatedValues(ProblemData.TrainingIndizes);
      }
    }

    public virtual IEnumerable<double> EstimatedTestValues {
      get {
        return GetEstimatedValues(ProblemData.TestIndizes);
      }
    }

    public virtual IEnumerable<double> GetEstimatedValues(IEnumerable<int> rows) {
      return Model.GetEstimatedValues(ProblemData, rows);
    }

    public IEnumerable<double> Thresholds {
      get {
        return (DoubleArray)this[ThresholdsResultsName].Value;
      }
    }

    public IEnumerable<double> EstimatedClassValues {
      get {
        return GetEstimatedClassValues(Enumerable.Range(0, ProblemData.Dataset.Rows));
      }
    }

    public IEnumerable<double> EstimatedTrainingClassValues {
      get {
        return GetEstimatedClassValues(ProblemData.TrainingIndizes);
      }
    }

    public IEnumerable<double> EstimatedTestClassValues {
      get {
        return GetEstimatedClassValues(ProblemData.TestIndizes);
      }
    }

    public IEnumerable<double> GetEstimatedClassValues(IEnumerable<int> rows) {
      return Model.GetEstimatedClassValues(ProblemData, rows);
    }

    #endregion
    #region events
    private void thresholds_ItemChanged(object sender, EventArgs<int> e) {
      OnThresholdsChanged(EventArgs.Empty);
    }

    private void thresholds_Reset(object sender, EventArgs e) {
      OnThresholdsChanged(EventArgs.Empty);
    }

    public event EventHandler ThresholdsChanged;
    private void OnThresholdsChanged(EventArgs e) {
      var listeners = ThresholdsChanged;
      if (listeners != null) listeners(this, e);
    }
    #endregion
  }
}
